/* This file is part of Roboarm.
 *
 * Roboarm is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Roboarm is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Roboarm; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package ch.educeth.roboarm;

import java.awt.*;
import java.awt.geom.*;
import java.util.*;

/**
 * Klasse implementiert die Darstellung des (alpha, beta)-Koordinatenraumes.
 */
public class ABPanel extends CoordinatePanel {
   public ABPanel(DataModel data, int w, int h) {
      super(data, w, h,"Alpha-Beta-Koordinaten");

      oWidthAB=120; oHeightAB=h-30;
      offsetWidth=120; offsetHeight=h-30;

      personalDelay=1.0;

      pos1=(int)(Math.toDegrees(data.getAlpha())+oWidthAB);
      pos2=(int)(oHeightAB-Math.toDegrees(data.getBeta()));

      allowedAB=new Area(new Rectangle2D.Double(oWidthAB+1,oHeightAB-179 ,179,179));
      forbiddenAB=new Area(new Rectangle2D.Double(0,0,w,h));
      forbiddenAB.subtract(allowedAB);

      initGeneralPath(goals);

      myC=new SimpleChart(new Rectangle(120, 90, 180, 180),180, 0, 180, 0,30,30,2,2);
   }

   public boolean isPointAllowed(double x, double y) {
      return !(forbiddenAB.contains(x,y) || shadowsAB.contains(x,y));
   }

   public boolean isLineAllowed(double x1, double y1, double x2, double y2) {
      GeneralPath p=new GeneralPath();
      p.moveTo((float)x1, (float) y1);
      p.lineTo((float)x2, (float)y2);
      p.lineTo((float)x2, (float)(y2-0.1));
      p.lineTo((float)x1, (float)(y1-0.1));
      p.lineTo((float)x1, (float)y1);

      Area ar=new Area(p);
      Area a=new Area((Shape)forbiddenAB); a.intersect(ar);
      Area b=new Area((Shape)shadowsAB); b.intersect(ar);

      return a.isEmpty() && b.isEmpty();
   }

   public void paintComponent(Graphics g){
      Graphics2D g2=(Graphics2D) g;

      super.paintComponent(g2);

      myC.drawGrid(g2);

      g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
      if (drawPath) drawPath(g2);
      g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_OFF);
      if (drawGoals) drawGoals(g2);

      g2.setStroke(basicStroke);
      g2.setColor(Color.red);
      g2.drawLine(pos1-10,pos2, pos1+10, pos2);
      g2.drawLine(pos1,pos2+10, pos1, pos2-10);

      g2.setPaint(shadowColor);
      g2.fill(shadowsAB);

      g2.setFont(f);

      g2.setColor(Color.blue);
      g2.drawString("Alpha",(int)oWidthAB+190,(int)oHeightAB+5);
      g2.drawString("0<Alpha<180",15,130);

      g2.setColor(Color.magenta);
      g2.drawString("Beta",(int)oWidthAB-10,(int)oHeightAB-190);
      g2.drawString("0<Beta<180",15,150);

      g2.setPaint(Color.gray);
      g2.drawRect(10, 116, 75, 40);
   }

   public void update(Observable o, Object i) {
      DataModel data=(DataModel) o;
      Message msg=(Message)i;

      if (i!=null) {
         counter=0;
         if (msg.startOfAnimation) initGeneralPath(path);
      }

      pos1=(int)(Math.toDegrees(data.getAlpha())+oWidthAB);
      pos2=(int)(oHeightAB-Math.toDegrees(data.getBeta()));

      if (drawPath)
         if((msg!=null && msg.isVertice) || (counter%25==0 && !AnimationLoop.isAB))
            path.lineTo(pos1, pos2);

      if (counter%((int) (personalDelay*AnimationLoop.delay))==0) paintImmediately(oWidthAB,oHeightAB-180,180, 180);

      counter++;
   };

   public void updateDataModel(int a, int b) {
      AnimationLoop.data.setAngles(Math.toRadians(a),Math.toRadians(b));
   }

   public double getXCoordinate(int s, int t) {
      return AnimationLoop.data.newXCoordinate(Math.toRadians(s),Math.toRadians(t));
   };

   public double getYCoordinate(int s, int t) {
      return AnimationLoop.data.newYCoordinate(Math.toRadians(s),Math.toRadians(t));
   }
}
