/* This file is part of Roboarm.
 *
 * Roboarm is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Roboarm is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Roboarm; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package ch.educeth.roboarm;

import java.util.*;

/**
 * Der AnimationsLoop. In dieser Klasse werden alle die Animation betreffenden Informationen
 * gespeichert: Zustand der Animation, gewaehlte Animationsart, abzulaufende Ziele, Animationsgeschwindigkeit usw.
 * Der Animationsthread laeuft schlussendlich auf Subklassen dieser Klasse. Jeweils nur eine Subklasse ist
 * die jetzige 'Animation' (Singleton-Pattern').
 */
public abstract class AnimationLoop implements Runnable{
   private static AnimationLoop anim;
   public static double delta=Math.PI/720;

   private static double oA;private static double oB;private static double oX;private static double oY;

/**
 * Datenmodell. Wird waehrend der Animation laufend aktualisiert.
 */
   public static DataModel data;
   private static Thread th;
   public static int delay=(int)(Math.pow(7.5,2));
   public static int estTimeSteps=0;

/**
 * Liste der Ziele, die der Animationsloop noch ablaufen muss
 */
   public static ArrayList goals=new ArrayList(1);

/**
 * Zustandsinformation ueber die jetzige Animation
 */
   public static boolean isAB=true;public static boolean isLine=true;

   public static AnimationLoop getInstance() { return anim; }

   public static void createSingleInstance(String s) {
      deleteGoals();

      try { anim=(AnimationLoop) (Class.forName(s).newInstance());
      } catch (Exception e) {anim=null;}
   }

   public static void stopAnimation() {
      if(isRunning()) th.stop();
      deleteGoals();
   }

   public static boolean isRunning() {
      return th!=null && th.isAlive();
   }

   public static void adjustSpeed(double value) {
      delay=(int)(Math.pow(1.5*value,2));
   }

   public static void startAnimation() {
      data.setChanged();
      data.notifyObservers(new Message(true,false,  true));
      th=new Thread(anim);
      th.start();
   }

   public static void verticeAnimation() {
      data.setChanged();
      data.notifyObservers(new Message(false, false, true));
   }

   public static void endOfAnimation() {
      data.setChanged();
      data.notifyObservers(new Message(false, true, true));
      CoordinatePanel.drawGoals=false;
      BasicHandler.changeButtons(true);
      stopAnimation();
   }

/**
 * Hinzufuegen eines Ziels, spezifiert in den (Alpha,Beta)-Koordinaten
 */
   public static void addGoalAB(double alpha, double beta) {
      double a=Math.toRadians(alpha), b=Math.toRadians(beta);
      int runs;

      if(!goals.isEmpty()) runs=calculateRunsAB(a-oA, b-oB);
      else runs=calculateRunsAB(a-data.getAlpha(), b-data.getBeta());

      estTimeSteps+=runs;

      Goal g=new AnimationLoop.Goal(a, b, data.newXCoordinate(a, b),data.newYCoordinate(a, b), runs);
      goals.add(g);

      oA=a; oB=b;
   }

/**
 * Hinzufuegen eines Ziels, spezifiert in den (x,y)-Koordinaten
 */
   public static void addGoalXY(double x, double y) {
      int runs;

      if(!goals.isEmpty()) runs=calculateRunsXY(oX, oY, x-oX, y-oY);
      else runs=calculateRunsXY(data.getX(), data.getY(), x-data.getX(), y-data.getY());

      estTimeSteps+=runs;

      Goal g=new AnimationLoop.Goal(data.newAlphaAngle(x, y),data.newBetaAngle(x, y),x,y, runs);
      goals.add(g);

      oX=x; oY=y;
   }

   public static void deleteGoals() {
      goals=new ArrayList(1);
      estTimeSteps=0;
   }

/**
 * Berechnen der Zeitschritte, die gebraucht werden um ein bestimmtes (alpha,beta)-Ziel zu erreichen.
 */
   protected static int calculateRunsAB(double deltaA, double deltaB) {
      return Math.max(Math.abs((int)(deltaA/delta)), Math.abs((int)(deltaB/delta)));
   }

/**
 * Berechnen der Zeitschritte, die gebraucht werden um das (x,y)-Ziel mit den Koordinaten (sX+deltaX, sY+deltaY)
 * vom Punkt (sX, sY) aus zu erreichen.
 */
   protected static int calculateRunsXY(double sX, double sY, double deltaX, double deltaY) {
      int runs=0;
      double dX=deltaX/10, dY=deltaY/10;

      for(int i=1; i<=10; i++) {
         int runsA=Math.abs((int)((data.newAlphaAngle(sX+(i-1)*dX, sY+(i-1)*dY)-data.newAlphaAngle(sX+i*dX, sY+i*dY))/delta));
         int runsB=Math.abs((int)((data.newBetaAngle(sX+(i-1)*dX, sY+(i-1)*dY)-data.newBetaAngle(sX+i*dX, sY+i*dY))/delta));
         runs+=(int)Math.max(runsA, runsB);
      }

      return runs;
   }

/**
 * Diese Klasse repraesentiert ein zu erreichendes Ziel
 */
   public static class Goal {
      public double alpha;public double beta;public double x;public double y;
      public int runs;

      public Goal(double a, double b, double x, double y, int r){
         alpha=a;
         beta=b;
         this.x=x;
         this.y=y;
         runs=r;
      }
   }
}
