/* This file is part of Roboarm.
 *
 * Roboarm is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Roboarm is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Roboarm; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package ch.educeth.roboarm;

import java.awt.event.*;
import javax.swing.*;
import java.util.*;

/**
 * Superklasse der Handler, die auf Inputs im AB- oder im XYPanel reagieren. Bietet diesen
 * Handlern verschiedene Services an.
 */
public abstract class BasicHandler extends MouseAdapter implements MouseMotionListener {
   protected static CoordinatePanel cXY;protected static CoordinatePanel cAB;

/**
 * zur Zeit installierter Handler (Singleton-Pattern
 */
   private static BasicHandler h;
   protected static JTextField info;
   public static ButtonGroup group;
   protected static JRadioButton lineAB;protected static JRadioButton seqAB;protected static JRadioButton lineXY;protected static JRadioButton seqXY;
   public static JButton reset;

   protected static boolean obstacleMode;
   private static boolean firstXlaterY;

   public static void setPanel(CoordinatePanel c1, CoordinatePanel c2) {
      cXY=c1; cAB=c2;
   }

   protected static void resetAnimation() {
      seqAB.setEnabled(false);
      seqXY.setEnabled(false);
      if(AnimationLoop.isAB) lineAB.setSelected(true); else lineXY.setSelected(true);
      AnimationLoop.isLine=true;
   }

   public static BasicHandler getHandlerInstance() { return h; }

/**
 * @param  s  Name der Klasse, die als neuer Handler fuer das XY- und das ABPanel installiert
 *            werden soll
 */
   public static void createHandlerInstances(String s) {
      try {
         cAB.removeMouseListener(h);
         cAB.removeMouseMotionListener(h);
         cXY.removeMouseListener(h);
         cXY.removeMouseMotionListener(h);

         h=(BasicHandler) Class.forName(s).newInstance();

         cAB.addMouseListener(h);
         cAB.addMouseMotionListener(h);
         cXY.addMouseListener(h);
         cXY.addMouseMotionListener(h);
      } catch(Exception e) {System.out.println("Exception in BasicHandler"+e.toString());}
   }

/**
 * Methode testet, ob vom jetzigen Standort des Roboterarms aus ein neuer Punkt erreichbar
 * ist. Dies ist ziemlich komplex, denn die gewaehlte Fortbewegungsart ist unabhaengig davon,
 * in welchem Fenster der Click stattgefunden hat (z.B. Fortbewegung Linie in (x, y), aber
 * Click fand im ABPanel statt)
 * @param  nX  x-Koordinate des zu pruefenden Punktes
 * @param  nY  y-Koordinate des zu pruefenden Punktes
 * @param  nA  alpha-Koordinate des zu pruefenden Punktes
 * @param  nB  beta-Koordinate des zu pruefenden Punktes
 */
   protected static boolean isAllowed(double nX, double nY, double nA, double nB) {
      if(AnimationLoop.isAB) {
         double oldA=Math.toDegrees(AnimationLoop.data.getAlpha())+cAB.offsetWidth;
         double oldB= cAB.offsetHeight-Math.toDegrees(AnimationLoop.data.getBeta());
   return cAB.isLineAllowed(oldA, oldB, nA+cAB.offsetWidth, cAB.offsetHeight-nB);
      } else {
         double oldX=AnimationLoop.data.getX()+cXY.offsetWidth;
         double oldY=cXY.offsetHeight-AnimationLoop.data.getY();
         double newX=nX+cXY.offsetWidth;
         double newY=cXY.offsetHeight-nY;

         if (AnimationLoop.isLine) return cXY.isLineAllowed(oldX, oldY, newX, newY);
         else {
            boolean b1=cXY.isLineAllowed(oldX, oldY, newX, oldY);
            boolean b2=cXY.isLineAllowed(newX, oldY, newX, newY);
            boolean b3=cXY.isLineAllowed(oldX, oldY, oldX, newY);
            boolean b4=cXY.isLineAllowed(oldX, newY, newX, newY);
            firstXlaterY=(b1&&b2);

            return ((b1 && b2) || (b3 && b4));
         }
      }
   }

/**
 * Hinzufuegen eines neuen Ziels. Hier findet eine dopplete Indirektion ueber BasicHandler und
 * ein CoordinatePanel (ABPanel oder XYPanel) statt, aus aehnlichen Gruenden wie schon bei der
 * Methode isAllowed vermerkt
 * @param  nX  x-Koordinate des hinzuzufuegenden Ziels
 * @param  nY  y-Koordinate des hinzuzufuegenden Ziels
 * @param  nA  alpha-Koordinate hinzuzufuegenden Ziels
 * @param  nB  beta-Koordinate hinzuzufuegenden Ziels
 */
   protected static void addGoals(double nX, double nY, double nA, double nB) {
      if (AnimationLoop.isAB ) {
         if(!AnimationLoop.isLine) cAB.addGoal(nA, cAB.offsetHeight-cAB.pos2);
         cAB.addGoal(nA,nB);
      } else {
         if(!AnimationLoop.isLine)
           if(firstXlaterY) cXY.addGoal(nX, cXY.offsetHeight-cXY.pos2);
           else cXY.addGoal(cXY.pos1-cXY.offsetWidth, nY);
         cXY.addGoal(nX,nY);
      }
   }

/**
 * Aktivieren/Desaktivieren der Buttons der Bewegungsauswahl, je nachdem, ob die Animation
 * beginnt oder zu Ende ist.
 */
   public static void changeButtons(boolean bool) {
      Enumeration e=group.getElements();
      while(e.hasMoreElements()) {
         JRadioButton b=(JRadioButton) e.nextElement();
         if (b.getText().startsWith("L")) b.setEnabled(bool);
   else b.setEnabled(bool && (!obstacleMode));
      }
      reset.setEnabled(bool);
   }
}
