/* This file is part of Roboarm.
 *
 * Roboarm is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Roboarm is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Roboarm; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package ch.educeth.roboarm;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.geom.*;

/**
 * Eine einfache Borderklasse. Am oberen Rand wird ein Bereich gefuellt und der
 * Titel wird dort abgebildet. Der sonstige Rand besteht aus einer schwarzen Linie
 * mit 1 Pixel Breite.
 */
public class BigTitledBorder extends AbstractBorder {
   private String title;
   private Font titleFont=new Font("Arial", Font.PLAIN, 20);
   private Color fillColor=new Color(200,200,200);
   private Color fontColor=Color.white;
   private Stroke outline=new BasicStroke(1.0f);
   private int insetFromTop=31;
   private int fontOffsetFromTop;

   public BigTitledBorder(String t) {
      title=t;
      fontOffsetFromTop=(titleFont.getSize()+insetFromTop)/2-2;
   }

/**
 * @param   f      Schriftart des Titels
 * @param   fill   Farbe mit der das Rechteck am oberen Rand, indem sich der Titel befindet, gefuellt wird
 * @param   text   Farbe des Titels (Schrift)
 * @param   t      Titel-String
 * @param   iFT    Hoehe des Rechtecks am oberen Rand
 */
   public BigTitledBorder(Font f, Color fill, Color text, String t, int iFT) {
      titleFont=f;
      fillColor=fill;
      fontColor=text;
      insetFromTop=iFT;
      title=t;
      insetFromTop=iFT;
      fontOffsetFromTop=(titleFont.getSize()+insetFromTop)/2-2;
   }

   public void paintBorder(Component c, Graphics g, int x, int y, int width, int height) {
      ((Graphics2D)g).setStroke(outline);
      g.setColor(Color.black);
      g.drawRect(x, y, width-1, height-1);

      g.setColor(fillColor);
      g.fillRect(x+1,y+1, width-2, insetFromTop-1);

      g.setColor(fontColor);
      g.setFont(titleFont);
      g.drawString(title,10,fontOffsetFromTop);
   }

   public Insets getBorderInsets(Component c) {return new Insets(insetFromTop, 1,1,1);}

   public boolean isBorderOpague() { return true; }
}