/* This file is part of Roboarm.
 *
 * Roboarm is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Roboarm is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Roboarm; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package ch.educeth.roboarm;

import java.util.*;

/**
 * Diese Klasse beinhaltet das dem gesamten Applet zugrundeliegende Datenmodell, und implementiert
 * die Umrechnung zwischen (x,y)- und (alpha,beta)-Koordinaten.
 */
public class DataModel extends Observable{
   private double alpha;
   private double beta;
   private double l1;
   private double l2;
   private double xCoord;
   private double  yCoord;

/**
 * Initialisierung des Datenmodells.
 * @param  a    Winkel alpha
 * @param  b    Winkel beta
 * @param  L1   Laenge des ersten Glieds des Roboterarms
 * @param  L2   Laenge des zweiten Glieds des Roboterarms
 */
   public DataModel(double a, double b, double L1, double L2) {
      alpha=a; beta=b;
      l1=L1; l2=L2;
      xCoord=newXCoordinate(a, b);
      yCoord=newYCoordinate(a, b);
   }

   public double getAlpha() {return alpha;}
   public double getBeta() {return beta;}

   public double getX() {return xCoord;}
   public double getY() {return yCoord;}

   public double getL1() {return l1;}
   public double getL2() {return l2;}

   public double newXCoordinate(double a, double b) {
      return l1*Math.cos(a)-l2*Math.cos(a+b);
   }

   public double newYCoordinate(double a, double b) {
      return l1*Math.sin(a)-l2*Math.sin(a+b);
   }

   public double newAlphaAngle(double x, double y) {
      double len=Math.sqrt(x*x+y*y), phi=Math.acos(x/len);

      if(y<0.0) phi*=-1;
      return phi+Math.acos((l1*l1-l2*l2+len*len)/(2*len*l1));
   }

   public double newBetaAngle(double x, double y) {
      return Math.acos((l1*l1+l2*l2-x*x-y*y)/(2*l1*l2));
   }

   public void setChanged() {super.setChanged();}

   public void setAngles(double a, double b) {
      alpha=a;
      beta=b;
      xCoord=newXCoordinate(alpha, beta);
      yCoord=newYCoordinate(alpha, beta);

      setChanged();
      notifyObservers();
   }

   public void setCoordinates(int x, int y) {
      xCoord=x;
      yCoord=y;
      alpha=newAlphaAngle(xCoord, yCoord);
      beta=newBetaAngle(xCoord, yCoord);

      setChanged();
      notifyObservers();
   }
}
