/* This file is part of Roboarm.
 *
 * Roboarm is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Roboarm is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Roboarm; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package ch.educeth.roboarm;

import java.awt.geom.*;

/**
 * Diese Klasse enthaelt alle Information ueber eingezeichnete Hindernisse
 */

public class ObstacleModule {
/**
 * von den Hindernissen im (x,y)-Raum belegte Flaeche
 */
   public Area obstacles=new Area();

/**
 * von den Hindernissen im (x,y)-Raum abgeschatteter Bereich
 */
   public GeneralPath shadowsXY=new GeneralPath();

/**
 * von den Hindernissen im (alpha,beta)-Raum abgeschatteter Bereich
 */
   public GeneralPath shadowsAB=new GeneralPath();

   private double distC;private double alphaC;private double l1;private double l2;private double alpha;private double beta;private double x;private double y;private double r;
   private DataModel data;
   private int oWAB;private int oHAB;private int oWXY;private int oHXY;

   public ObstacleModule(int aW, int aH, int xW, int xH) {
      oWAB=aW; oHAB=aH;
      oWXY=xW; oHXY=xH;
   }

/** Methode zum Hinzufuegen eines Kreises im (x,y)-Raum als Hindernis
  * @param  x      x-Koordinate des Kreismittelpunkts
  * @param  y      y-Koordinate des Kreismittelpunkts
  * @param  r      Radius des Kreises
  * @param isLeft  Wahr, falls der Roboterarm sich 'links' des Kreises befindet
  */
   public void addAsObstacle(double x, double y, double r, boolean isLeft) {
      Arc2D.Double arc1=new Arc2D.Double(), arc2=new Arc2D.Double();
      data=new DataModel(Math.PI/2, Math.PI/2, AnimationLoop.data.getL1(), AnimationLoop.data.getL2());

      distC=Math.sqrt(x*x+y*y);
      alphaC=Math.acos(x/distC);
      if(y<0) alphaC*=-1;

      double delta=Math.asin(r/distC);

      l1=data.getL1(); l2=data.getL2();
      this.x=x; this.y=y; this.r=r;

      obstacles.add(new Area(new Ellipse2D.Double(oWXY+x-r, oHXY-y-r, 2*r, 2*r)));

      if((distC>=r+l1) || y<-r) {         // Hindernis blockiert den Roboterarm nicht
         alpha=Math.max(0, alphaC+delta);

         double curDist=Math.sqrt((l1*Math.cos(alpha)-x)*(l1*Math.cos(alpha)-x)+(l1*Math.sin(alpha)-y)*(l1*Math.sin(alpha)-y));
         double gamma=Math.acos((l1*l1+curDist*curDist-distC*distC)/(2*l1*curDist));

   beta=gamma+Math.asin(r/curDist);

         shadowsAB.moveTo((float)(oWAB+Math.toDegrees(alpha)), (float)(oHAB-Math.toDegrees(beta)));

   if(y<r && x>l1) {
      shadowsXY.moveTo((float)(oWXY+l1+l2+5), (float)(oHXY-data.newYCoordinate(alpha, beta)));
      shadowsXY.lineTo((float)(oWXY+data.newXCoordinate(alpha, beta)),(float)(oHXY-data.newYCoordinate(alpha, beta)));
   } else {
      shadowsXY.moveTo((float)(oWXY+data.newXCoordinate(alpha, beta)),(float)(oHXY-data.newYCoordinate(alpha, beta)));
         }

         calculateTangentialPath(alphaC+Math.acos((l1*l1+distC*distC-(l2+r)*(l2+r))/(2*l1*distC)),1);
         calculateTangentialPath(Math.max(0,alphaC-delta), -1);

   if(y<-r) {
      double de=Math.asin(r/Math.sqrt((l1-x)*(l1-x)+y*y));
      arc1.setArcByCenter(oWXY+l1, oHXY, l2, Math.toDegrees(Math.PI+gamma-de), 2*Math.toDegrees(de), Arc2D.OPEN);
      shadowsXY.append(arc1, true);
   } else if(y<r && x>l1) {
          shadowsXY.lineTo((float)shadowsXY.getCurrentPoint().getX(), oHXY+70);
      shadowsXY.lineTo((float)(oWXY+l1+l2+5), oHXY+70);
   } else {
      arc1.setArcByCenter(oWXY,oHXY,l1+l2,Math.toDegrees(alphaC-delta),2*Math.toDegrees(delta), Arc2D.OPEN);
      shadowsXY.append(arc1, true);
   }

   shadowsAB.lineTo((float)(oWAB+Math.toDegrees(alphaC-delta)), oHAB-180);
      } else {     // Hindernis blockiert den Roboterarm...
         boolean isFullyBlocking=distC*distC-r*r<=l1*l1;   // ... mit gesamten Kreisdurchmesser??

   if(isLeft) {
      if(isFullyBlocking) {
         alpha=alphaC+delta;
               beta=0;
      } else {
               delta=Math.acos((l1*l1+distC*distC-r*r)/(2*l1*distC));
               alpha=alphaC+delta;
      }

            shadowsAB.moveTo(oWAB, oHAB);
      shadowsAB.lineTo((float)(oWAB+Math.toDegrees(alpha)), oHAB);

            shadowsXY.moveTo((float)(oWXY+l1+l2+5), (float)(oHXY-l1-l2-5));
            shadowsXY.lineTo((float)(oWXY+l1+l2+5), oHXY+70);
      shadowsXY.lineTo((float)(oWXY+(l1-l2)*Math.cos(alpha)), oHXY+70);
      shadowsXY.lineTo((float)(oWXY+(l1-l2)*Math.cos(alpha)),(float)(oHXY-(l1-l2)*Math.sin(alpha)));

            double theta=Math.acos((l1*l1+r*r-distC*distC)/(2*l1*r))-Math.PI/2;
            arc1.setArcByCenter(oWXY+l1*Math.cos(alpha), oHXY-l1*Math.sin(alpha),l2,180+Math.toDegrees(alpha), Math.toDegrees(theta),Arc2D.OPEN);
            if(!(distC*distC-r*r<=l1*l1)) shadowsXY.append(arc1,false);

      calculateTangentialPath(alphaC+Math.acos((l1*l1+distC*distC-(l2+r)*(l2+r))/(2*l1*distC)),-1);
      calculateTangentialPath(alphaC+Math.asin(r/distC), 1);

            if(isFullyBlocking) {
         arc2.setArcByCenter(oWXY+l1*Math.cos(alpha), oHXY-l1*Math.sin(alpha), l2,180+Math.toDegrees(alpha+beta), 180-Math.toDegrees(beta),Arc2D.OPEN);
         shadowsXY.append(arc2,false);
            }

            shadowsXY.lineTo((float)shadowsXY.getCurrentPoint().getX(), (float)(oHXY-l1-l2-5));
      shadowsXY.lineTo((float)(oWXY+l1+l2+5), (float)(oHXY-l1-l2-5));

      shadowsAB.lineTo((float)(oWAB+Math.toDegrees(alpha)), oHAB-180);
      shadowsAB.lineTo(oWAB, oHAB-180);
      shadowsAB.lineTo(oWAB, oHAB);
   } else {
      double oldDelta=0, theta;

      if(!isFullyBlocking) {
         oldDelta=delta;
         delta=Math.acos((l1*l1+distC*distC-r*r)/(2*l1*distC));
      }

      alpha=alphaC-delta;

      shadowsAB.moveTo(oWAB+180, oHAB-180);
      shadowsAB.lineTo(oWAB+180, oHAB);
      shadowsAB.lineTo((float)(oWAB+Math.toDegrees(alpha)), oHAB);

            shadowsXY.moveTo((float)(oWXY-l1-l2-5), oHXY+70);
            shadowsXY.lineTo((float)(oWXY+(l1-l2)*Math.cos(alpha)), oHXY+70);
      shadowsXY.lineTo((float)(oWXY+(l1-l2)*Math.cos(alpha)),(float)(oHXY-(l1-l2)*Math.sin(alpha)));

      if(!isFullyBlocking) {
         double curDist=Math.sqrt((l1*Math.cos(alpha)-x)*(l1*Math.cos(alpha)-x)+(l1*Math.sin(alpha)-y)*(l1*Math.sin(alpha)-y));
         theta=Math.toDegrees(1.5*Math.PI-Math.acos((l1*l1+curDist*curDist-distC*distC)/(2*l1*curDist)));
      } else theta=180;

      arc1.setArcByCenter(oWXY+l1*Math.cos(alpha), oHXY-l1*Math.sin(alpha),l2,180+Math.toDegrees(alpha), theta,Arc2D.OPEN);
      shadowsXY.append(arc1,false);

      shadowsAB.lineTo((float)(oWAB+Math.toDegrees(alpha)), (float)(oHAB-Math.toDegrees(theta)));

      if(!isFullyBlocking) {
         calculateTangentialPath(alpha+delta-oldDelta,1);
         shadowsAB.lineTo((float)(oWAB+Math.toDegrees(alpha)), oHAB-180);
      }

            shadowsAB.lineTo(oWAB+180, oHAB-180);
      shadowsXY.lineTo((float)shadowsXY.getCurrentPoint().getX(), (float)(oHXY-l1-l2-5));
      shadowsXY.lineTo((float)(oWXY-l1-l2-5), (float)(oHXY-l1-l2-5));
      shadowsXY.lineTo((float)(oWXY-l1-l2-5), oHXY+70);
         }
      }
   }

   private void calculateTangentialPath(double endA, int betaAdd) {
      int runs=(int)Math.abs((endA-alpha)/(Math.PI/180));
      double d=(endA-alpha)/runs;

      for(int k=0; k<runs-1; k++) {
         alpha+=d;

         double curDist=Math.sqrt((l1*Math.cos(alpha)-x)*(l1*Math.cos(alpha)-x)+(l1*Math.sin(alpha)-y)*(l1*Math.sin(alpha)-y));
         double gamma=Math.acos((l1*l1+curDist*curDist-distC*distC)/(2*l1*curDist));

         if(curDist*curDist-r*r<=l2*l2) {
            if(alpha<alphaC) beta=2*Math.PI-gamma-Math.asin(r/curDist);
            else beta=gamma+betaAdd*Math.asin(r/curDist);
         } else {
            if(alpha<alphaC) beta=2*Math.PI-gamma-Math.acos((curDist*curDist+l2*l2-r*r)/(2*l2*curDist));
            else beta=gamma+betaAdd*Math.acos((curDist*curDist+l2*l2-r*r)/(2*l2*curDist));
         }

         shadowsAB.lineTo((float)(oWAB+Math.toDegrees(alpha)), (float)(oHAB-Math.toDegrees(beta)));
         shadowsXY.lineTo((float)(oWXY+data.newXCoordinate(alpha, beta)), (float)(oHXY-data.newYCoordinate(alpha, beta)));
      }
   }
}
