/* This file is part of Roboarm.
 *
 * Roboarm is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Roboarm is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Roboarm; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package ch.educeth.roboarm;

import java.awt.*;
import java.util.*;
import javax.swing.*;

/**
 * Diese Klasse ist das Status-Panel. Sie ist ebenfalls ein Observer des Datenmodells
 * und wird waehrend der Animation notifiziert (speichert durchlaufene Winkel und
 * Zeitschritte)
 */
public class Status extends JPanel implements Observer {
    private JTextField timeField;private JTextField alphaField;private JTextField betaField;
    private double alpha;private double beta;private double deltaAlpha;private double deltaBeta;
    private int time;

    public Status(DataModel data, Color background) {
       setBackground(background);
       setBorder(BorderFactory.createTitledBorder("Status"));

       setMinimumSize(new Dimension(200,85));
       setMaximumSize(new Dimension(200,85));
       setPreferredSize(new Dimension(200,85));

       data.addObserver(this);

       JLabel timeLabel = new JLabel("Zeitschritte");
       timeLabel.setBackground(background);
       JLabel alphaLabel = new JLabel("durchl. Alpha");
       alphaLabel.setBackground(background);
       JLabel betaLabel = new JLabel("durchl. Beta");
       betaLabel.setBackground(background);


       timeField=new JTextField(8);
       timeField.setEditable(false);
       timeField.setBackground(background);


       alphaField=new JTextField(8);
       alphaField.setEditable(false);
       alphaField.setBackground(background);

       betaField=new JTextField(8);
       betaField.setEditable(false);
       betaField.setBackground(background);


       JPanel labelPane = new JPanel(new GridLayout(0, 1));
       labelPane.add(timeLabel);
       labelPane.add(alphaLabel);
       labelPane.add(betaLabel);
       labelPane.setBackground(background);

       JPanel fieldPane = new JPanel(new GridLayout(0, 1));
       fieldPane.add(timeField);
       fieldPane.add(alphaField);
       fieldPane.add(betaField);
       fieldPane.setBackground(background);

       setLayout(new BorderLayout());
       add(labelPane, BorderLayout.CENTER);
       add(fieldPane, BorderLayout.EAST);
   }

   public void update(Observable o, Object i) {
      DataModel d=(DataModel)o;
      Message msg=(Message)i;

      if (i!=null) {
         alphaField.setEnabled(msg.endOfAnimation);
         betaField.setEnabled(msg.endOfAnimation);
         timeField.setEnabled(msg.endOfAnimation);

         if (msg.startOfAnimation) {
            time=0;
            deltaAlpha=0.0;
            deltaBeta=0.0;
         } else if (msg.endOfAnimation) {
            alphaField.setText(Integer.toString((int)Math.toDegrees(deltaAlpha))+" Grad");
            betaField.setText(Integer.toString((int)Math.toDegrees(deltaBeta))+" Grad");
            timeField.setText(Integer.toString(time));
         }
      } else {
         time++;
         deltaAlpha+=Math.abs(alpha-d.getAlpha());
         deltaBeta+=Math.abs(beta-d.getBeta());

         alpha=d.getAlpha();
         beta=d.getBeta();
      }
   }
}




