/* This file is part of Roboarm.
 *
 * Roboarm is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Roboarm is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Roboarm; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package ch.educeth.roboarm;

import java.awt.geom.*;
import java.util.*;
import java.awt.*;

/**
 * Diese Klasse erstellt das Zeitdiagramm (zeitliche Veraenderung der Winkel Alpha und
 * Beta). TPanel ist ebenfalls ein Observer des Datenmodells.
 */
public class TPanel extends SimpleDrawablePanel {
   private double currentTime=0;private double timeAdvance;
   private int estTSteps;private int xScale;
   private GeneralPath alphaCurve=new GeneralPath();private GeneralPath betaCurve=new GeneralPath();

   public TPanel(DataModel data, int w, int h) {
      super(data, w, h,"Alpha-Beta-Zeitdiagramm");

      offsetWidth=120; offsetHeight=h-30;
      xScale=5; personalDelay=2.0;

      myC=new SimpleChart(new Rectangle(120, 90, 180, 180),xScale*6, 0, 180, 0,xScale,30,2,2);
      resetCurves();
   }

   public void paintComponent(Graphics g) {
      Graphics2D g2=(Graphics2D) g;


super.paintComponent(g2);
      myC.drawGrid(g2);

      g2.setFont(f);
      g2.setStroke(pathStroke);

      g2.setColor(Color.black);
      g2.drawString("Zeit",(int)offsetWidth+190,(int)offsetHeight+5);
      g2.drawString(",",(int)offsetWidth-4,(int)offsetHeight-190);

      g2.setColor(Color.blue);
      g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
      g2.draw(alphaCurve);
      g2.drawString("Alpha",(int)offsetWidth-35,(int)offsetHeight-190);

      g2.setColor(Color.magenta);
      g2.draw(betaCurve);

      g2.drawString("Beta",(int)offsetWidth,(int)offsetHeight-190);
   }

/**
 * zuruecksetzen der beiden Kurven; beim Animationsbeginn aufgerufen.
 */
   public void resetCurves() {
         alphaCurve.reset(); betaCurve.reset();
         alphaCurve.moveTo(offsetWidth, (float)(offsetHeight-Math.toDegrees(AnimationLoop.data.getAlpha())));
         betaCurve.moveTo(offsetWidth, (float)(offsetHeight-Math.toDegrees(AnimationLoop.data.getBeta())));
   }

   public void update(Observable o, Object i) {
      DataModel data=(DataModel) o;
      Message m=(Message) i;

      if (doNotDraw) return;

      if (m!=null && m.startOfAnimation) {
         currentTime=0;
counter=0;
         resetCurves();

         estTSteps=AnimationLoop.estTimeSteps;
         xScale=1+(int)Math.round(((double)estTSteps)/6);
         xScale+=10-xScale%10;
         timeAdvance=30.0/xScale;

         myC.setXSpacing(xScale,3);
         myC.setNewBoundsX(6*xScale, 0);
      } else if(m!=null) counter=0;

      alphaCurve.lineTo((float)(offsetWidth+currentTime+1),(float)(offsetHeight-Math.toDegrees(data.getAlpha())));
      betaCurve.lineTo((float)(offsetWidth+currentTime+1),(float)(offsetHeight-Math.toDegrees(data.getBeta())));

      if (counter%((int) (personalDelay*AnimationLoop.delay))==0) paintImmediately(0,0,getWidth(), getHeight());

      currentTime+=timeAdvance;
      counter++;
   }
}